@@ -2,48 +2,40 @@
 Provider of database access to other hooks.  All first-party hooks with database support will
 require the asynchronous database provider to be present.
 
-Synchronous
-~~~~~~~~~~~
+Asynchronous
+~~~~~~~~~~~~
+
+Dependencies:
+    `Tortoise <https://tortoise-orm.readthedocs.io>`_
+
+    Any database-specific librariess (e.g. asyncpg for PostgreSQL)
 
 Config:
     url (str):
-        Database connection string, as defined by Peewee's :ref:`db_url`.
+        Database connection string, as defined by Tortoise's :ref:`tortoise:db_url`.
 
-This hook provides persistent storage via a database.  Any database types supported by Peewee can
-be used, though the usual caveats apply: if a hook requires fields specific to a single database
-type, the app is effectively locked-in to that type.
+Synchronous
+~~~~~~~~~~~
 
-Hooks should subclass :class:`.BaseModel` for their data structures.  At startup, they can register
-their models to the database connection by calling :meth:`.DatabaseHook.add_models` (obtained from
-``host.resources[DatabaseHook]``), which will create any needed tables on first runs.
+.. deprecated:: 0.10.0
+    Use the asynchronous variant instead.
 
-.. note::
-    This hook requires the `Peewee <http://docs.peewee-orm.com>`_ Python module, along with any
-    database-specific libraries (e.g. Psycopg2 for PostgreSQL).
+Dependencies:
+    `Peewee <http://docs.peewee-orm.com>`_
+
+    Any database-specific libraries (e.g. Psycopg2 for PostgreSQL)
+
+Config:
+    url (str):
+        Database connection string, as defined by Peewee's :ref:`peewee:db_url`.
 
 .. warning::
     Database requests will block all other running tasks; notably, all plugs will be unable to make
     any progress whilst long-running queries are executing.
-
-Asynchronous
-~~~~~~~~~~~~
-
-Config:
-    url (str):
-        Tortoise `database connection string
-        <https://tortoise-orm.readthedocs.io/en/latest/databases.html>`_.
-
-This hook provides persistent storage via a database.  Any database types supported by Tortoise can
-be used.  At startup, they can register their models to the database connection by calling
-:meth:`.AsyncDatabaseHook.add_models` (obtained from ``host.resources[AsyncDatabaseHook]``), which
-will create any needed tables on first runs.
-
-.. note::
-    This hook requires the `Tortoise <https://tortoise-orm.readthedocs.io>`_ Python module, along
-    with any database-specific libraries (e.g. asyncpg for PostgreSQL).
 """
 
 import logging
+from warnings import warn
 
 try:
     from peewee import DatabaseProxy, Model
@@ -85,15 +77,25 @@ if Model:
 
 class DatabaseHook(immp.ResourceHook, _ModelsMixin):
     """
-    Hook that provides generic database access to other hooks, backed by :mod:`peewee`.  Because
-    models are in the global scope, they can only be attached to a single database, therefore this
-    hook acts as the single source of truth for obtaining a "global" database.
+    Hook that provides generic data access to other hooks, backed by :ref:`Peewee <peewee:api>`.
+    Because models are in the global scope, they can only be attached to a single database,
+    therefore this hook acts as the single source of truth for obtaining that "global" database.
+
+    Hooks should subclass :class:`.BaseModel` for their data structures, in order to gain the
+    database connection.  At startup, they can register their models to the database connection by
+    calling :meth:`add_models` (obtained from ``host.resources[DatabaseHook]``), which will create
+    any needed tables the first time models are added.
+
+    Any database types supported by Peewee can be used, though the usual caveats apply: if a hook
+    requires fields specific to a single database type, the system as a whole is effectively
+    locked-in to that type.
     """
 
     schema = immp.Schema({"url": str})
 
     def __init__(self, name, config, host):
         super().__init__(name, config, host)
+        warn("DatabaseHook is deprecated, migrate to AsyncDatabaseHook", DeprecationWarning)
         if not Model:
             raise immp.PlugError("'peewee' module not installed")
         self.db = None
@@ -116,9 +118,13 @@ class DatabaseHook(immp.ResourceHook, _ModelsMixin):
 
 class AsyncDatabaseHook(immp.ResourceHook, _ModelsMixin):
     """
-    Hook that provides generic database access to other hooks, backed by :mod:`tortoise`.  Because
+    Hook that provides generic data access to other hooks, backed by :mod:`tortoise`.  Because
     models are in the global scope, they can only be attached to a single database, therefore this
-    hook acts as the single source of truth for obtaining a "global" database.
+    hook acts as the single source of truth for obtaining that "global" database.
+
+    Hooks should register their models to the database connection at startup by calling
+    :meth:`add_models` (obtained from ``host.resources[AsyncDatabaseHook]``), which will create any
+    needed tables the first time models are added.
     """
 
     schema = immp.Schema({"url": str})
