@@ -5,6 +5,17 @@ one or more others.
 Sync
 ~~~~
 
+Requirements:
+    `Jinja2 <http://jinja.pocoo.org>`_:
+        Required to use ``name-format``.
+    `emoji <https://github.com/carpedm20/emoji/>`_:
+        Required to use ``strip-name-emoji``.
+
+Dependencies:
+    :class:`.AsyncDatabaseHook`:
+        If present, synced message IDs will be persisted to the database, allowing referenced
+        messages (replies, forwards, attachments) to be handled correctly across restarts.
+
 Config:
     channels ((str, str list) dict):
         Mapping from virtual channel names to lists of channel names to bridge.
@@ -55,6 +66,12 @@ synced channels.
 Forward
 ~~~~~~~
 
+Requirements:
+    `Jinja2 <http://jinja.pocoo.org>`_:
+        Required to use ``name-format``.
+    `emoji <https://github.com/carpedm20/emoji/>`_:
+        Required to use ``strip-name-emoji``.
+
 Config:
     channels ((str, str list) dict):
         Mapping from source channel names to lists of channel names to forward to.
@@ -89,10 +106,6 @@ Config:
 
 When a message is received in a configured source channel, a copy is pushed to all downstream
 channels.  Unlike a sync, this is a one-direction copy, useful for announcements or alerts.
-
-.. note::
-    Use of ``name-format`` requires the `Jinja2 <http://jinja.pocoo.org>`_ Python module.  Use of
-    ``strip-name-emoji`` requires the **emoji** Python module.
 """
 
 from asyncio import BoundedSemaphore, gather
@@ -451,7 +464,7 @@ class SyncPlug(immp.Plug):
     async def put(self, channel, msg):
         if channel.source in self._hook.config["channels"]:
             ref = await self._hook.send(channel.source, msg)
-            return [ref.key]
+            return [immp.Receipt(ref.key, channel)]
         else:
             raise immp.PlugError("Send to unknown sync channel: {}".format(repr(channel)))
 
@@ -585,9 +598,9 @@ class _SyncHookBase(immp.Hook):
 
     async def _send(self, channel, msg):
         try:
-            ids = await channel.send(msg)
-            log.debug("Synced IDs in %r: %r", channel, ids)
-            return (channel, ids)
+            receipts = await channel.send(msg)
+            log.debug("Synced IDs in %r: %r", channel, [receipt.id for receipt in receipts])
+            return (channel, receipts)
         except Exception:
             log.exception("Failed to relay message to channel: %r", channel)
             return (channel, [])
@@ -733,7 +746,9 @@ class SyncHook(_SyncHookBase):
         # need to wait for all plugs to complete and have their IDs cached before processing any
         # further messages.
         async with self._lock:
-            ids = dict(await gather(*queue))
+            all_receipts = dict(await gather(*queue))
+            ids = {channel: [receipt.id for receipt in receipts]
+                   for channel, receipts in all_receipts.items()}
             if ref:
                 ref.ids.update(ids)
             else:
