@@ -1,6 +1,9 @@
 """
 Connect to `Discord <https://discordapp.com>`_ as a bot.
 
+Requirements:
+    `discord.py <https://discordpy.readthedocs.io>`_
+
 Config:
     token (str):
         Discord token for the bot user.
@@ -28,12 +31,9 @@ an `incoming webhook <https://discordapp.com/developers/docs/resources/webhook>`
 each channel you intend to send messages to.  A new webhook can be created over the API, or in the
 UI via Edit Channel > Webhooks.  A fallback style incorporating the user's name in the message text
 will be used in lieu of a webhook, e.g. with direct messages.
-
-.. note::
-    This plug requires the `discord.py <https://discordpy.readthedocs.io>`_ Python module.
 """
 
-from asyncio import Condition, ensure_future
+from asyncio import Condition, ensure_future, gather
 from collections import defaultdict
 from datetime import timezone
 from functools import partial
@@ -119,9 +119,14 @@ class DiscordRichText(immp.RichText):
     _pre_regex = re.compile(r"```\n?(.+?)\n?```", re.DOTALL)
 
     _mention_regex = re.compile(r"<@!?(\d+)>")
+    _role_mention_regex = re.compile(r"<@&(\d+)>")
     _channel_regex = re.compile(r"<#(\d+)>")
     _emoji_regex = re.compile(r"<(:[^: ]+?:)\d+>")
 
+    @classmethod
+    def _sub_role_mention(cls, roles, match):
+        return "@{}".format(roles.get(int(match.group(1)), "&{}".format(match.group(1))))
+
     @classmethod
     def _sub_channel(cls, discord_, match):
         return "#{}".format(discord_._client.get_channel(int(match.group(1))).name)
@@ -194,6 +199,7 @@ class DiscordRichText(immp.RichText):
         segments = []
         points = list(sorted(changes.keys()))
         formatting = {}
+        roles = {role.id: role.name for role in message.role_mentions}
         # Iterate through text in change start/end pairs.
         for start, end in zip([0] + points, points + [len(plain)]):
             formatting.update(changes[start])
@@ -206,6 +212,7 @@ class DiscordRichText(immp.RichText):
             else:
                 part = emojize(plain[start:end], use_aliases=True)
                 # Strip Discord channel/emoji tags, replace with a plain text representation.
+                part = cls._role_mention_regex.sub(partial(cls._sub_role_mention, roles), part)
                 part = cls._channel_regex.sub(partial(cls._sub_channel, discord_), part)
                 part = cls._emoji_regex.sub(r"\1", part)
             segments.append(immp.Segment(part, **formatting))
@@ -307,7 +314,7 @@ class DiscordMessage(immp.Message):
     """
 
     @classmethod
-    def from_message(cls, discord_, message, edited=False, deleted=False):
+    async def from_message(cls, discord_, message, edited=False, deleted=False):
         """
         Convert a :class:`discord.Message` into a :class:`.Message`.
 
@@ -325,12 +332,16 @@ class DiscordMessage(immp.Message):
             .DiscordMessage:
                 Parsed message object.
         """
-        text = None
+        text = reply_to = None
         channel = immp.Channel(discord_, message.channel.id)
         user = DiscordUser.from_user(discord_, message.author)
         attachments = []
         if message.content:
             text = DiscordRichText.from_message(discord_, message, channel)
+        if message.reference:
+            receipt = immp.Receipt(message.reference.message_id,
+                                   immp.Channel(discord_, message.reference.channel_id))
+            reply_to = await discord_.get_message(receipt)
         for attach in message.attachments:
             if attach.filename.endswith((".jpg", ".png", ".gif")):
                 type_ = immp.File.Type.image
@@ -356,6 +367,7 @@ class DiscordMessage(immp.Message):
                                 deleted=deleted,
                                 text=text,
                                 user=user,
+                                reply_to=reply_to,
                                 attachments=attachments,
                                 raw=message)
 
@@ -428,18 +440,18 @@ class DiscordClient(discord.Client):
 
     async def on_message(self, message):
         log.debug("Received a new message")
-        self._plug.queue(DiscordMessage.from_message(self._plug, message))
+        self._plug.queue(await DiscordMessage.from_message(self._plug, message))
 
     async def on_message_edit(self, before, after):
         log.debug("Received an updated message")
         if before.content == after.content:
             # Text content hasn't changed -- maybe just a link unfurl embed added.
             return
-        self._plug.queue(DiscordMessage.from_message(self._plug, after, edited=True))
+        self._plug.queue(await DiscordMessage.from_message(self._plug, after, edited=True))
 
     async def on_message_delete(self, message):
         log.debug("Received a deleted message")
-        self._plug.queue(DiscordMessage.from_message(self._plug, message, deleted=True))
+        self._plug.queue(await DiscordMessage.from_message(self._plug, message, deleted=True))
 
 
 class DiscordPlug(immp.HTTPOpenable, immp.Plug):
@@ -551,13 +563,14 @@ class DiscordPlug(immp.HTTPOpenable, immp.Plug):
         dc_channel = self._get_channel(channel)
         dc_before = await dc_channel.fetch_message(before.id) if before else None
         history = dc_channel.history(before=dc_before, oldest_first=False)
-        messages = [DiscordMessage.from_message(self, message) async for message in history]
+        messages = await gather(*[DiscordMessage.from_message(self, message)
+                                  async for message in history])
         return list(reversed(messages))
 
     async def get_message(self, receipt):
         dc_channel = self._get_channel(receipt.channel)
         message = await dc_channel.fetch_message(receipt.id)
-        return DiscordMessage.from_message(self, message)
+        return await DiscordMessage.from_message(self, message)
 
     def _resolve_channel(self, channel):
         dc_channel = self._get_channel(channel)
@@ -571,6 +584,7 @@ class DiscordPlug(immp.HTTPOpenable, immp.Plug):
 
     async def _requests(self, dc_channel, webhook, msg):
         name = image = None
+        reply_to = reply_ref = reply_embed = None
         embeds = []
         files = []
         if msg.user:
@@ -602,14 +616,21 @@ class DiscordPlug(immp.HTTPOpenable, immp.Plug):
                 resolved = await self.resolve_message(attach)
                 embeds.append((await DiscordMessage.to_embed(self, resolved), "sent a message"))
         if msg.reply_to:
-            # Discord offers no reply mechanism, so instead we just fetch the referenced message
-            # and render it manually.
-            resolved = await self.resolve_message(msg.reply_to)
-            embeds.append((await DiscordMessage.to_embed(self, resolved, True), None))
-        if webhook:
+            if isinstance(msg.reply_to, immp.Receipt):
+                if msg.reply_to.channel.plug.network_id == self.network_id:
+                    reply_to = await self.resolve_message(msg.reply_to)
+                    reply_ref = reply_to.raw.to_reference()
+            if not reply_to:
+                reply_to = msg.reply_to
+            reply_embed = await DiscordMessage.to_embed(self, reply_to, True)
+        if webhook and msg.user:
             # Sending via webhook: multiple embeds and files supported.
             requests = []
             text = None
+            if reply_embed:
+                # Webhooks can't reply to other messages, quote the target in an embed instead.
+                # https://github.com/discord/discord-api-docs/issues/2251
+                embeds.append((reply_embed, None))
             if msg.text:
                 rich = msg.text.clone()
                 if msg.action:
@@ -633,19 +654,21 @@ class DiscordPlug(immp.HTTPOpenable, immp.Plug):
             # Sending via client: only a single embed per message.
             requests = []
             text = embed = None
+            lines = []
             rich = msg.render(link_name=False, edit=msg.edited) or None
             if rich:
-                lines = DiscordRichText.chunk_split(DiscordRichText.to_markdown(self, rich))
-                if len(lines) > 1:
-                    # Multiple messages required to accommodate the text.
-                    requests.extend(dc_channel.send(content=line) for line in lines)
-                else:
-                    text = lines[0]
+                text, *lines = DiscordRichText.chunk_split(DiscordRichText.to_markdown(self, rich))
+            if reply_embed and not reply_ref:
+                embeds.append((reply_embed, None))
             if len(embeds) == 1:
                 # Attach the only embed to the message text.
                 embed, _ = embeds.pop()
             if text or embed or files:
-                requests.append(dc_channel.send(content=text, embed=embed, files=files))
+                # Primary message: set reference for reply-to if applicable.
+                requests.append(dc_channel.send(content=text, embed=embed, files=files,
+                                                reference=reply_ref))
+            # Send the remaining text if multiple messages were required to accommodate it.
+            requests.extend(dc_channel.send(content=line) for line in lines)
             for embed, desc in embeds:
                 # Send any additional embeds in their own separate messages.
                 content = None
@@ -658,10 +681,14 @@ class DiscordPlug(immp.HTTPOpenable, immp.Plug):
     async def put(self, channel, msg):
         dc_channel, webhook = self._resolve_channel(channel)
         requests = await self._requests(dc_channel, webhook, msg)
-        sent = []
+        receipts = []
         for request in requests:
-            sent.append(await request)
-        return [str(resp.id) for resp in sent]
+            message = await request
+            if not message.channel:
+                # Webhook-sent messages won't have their channel set.
+                message.channel = dc_channel
+            receipts.append(await DiscordMessage.from_message(self, message))
+        return receipts
 
     async def delete(self, sent):
         dc_channel = self._resolve_channel(sent.channel)[0]
