@@ -1,6 +1,9 @@
 """
 Connect to `Google Hangouts <https://hangouts.google.com>`_ as a regular user.
 
+Requirements:
+    `hangups <https://hangups.readthedocs.io>`_
+
 Config:
     cookie (str):
         Path to a cookie text file read/written by :func:`hangups.get_auth_stdin`.
@@ -15,9 +18,6 @@ cookie file manually by interacting with hangups directly::
     $ python -m hangups.auth
 
 This will generate a cookie file called *refresh_token.txt* in the current directory.
-
-.. note::
-    This plug requires the `hangups <https://hangups.readthedocs.io>`_ Python module.
 """
 
 from asyncio import CancelledError, Condition, ensure_future, gather, sleep
@@ -760,7 +760,9 @@ class HangoutsPlug(immp.HTTPOpenable, immp.Plug):
             info = immp.Message(user=clone.user, action=True, text="forwarded a message")
             own_requests = await self._requests(conv, info)
         requests += own_requests
-        events = []
+        receipts = []
         for request in requests:
-            events.append(await self._client.send_chat_message(request))
-        return [event.created_event.event_id for event in events]
+            response = await self._client.send_chat_message(request)
+            event = hangups.conversation.Conversation._wrap_event(response.created_event)
+            receipts.append(await HangoutsMessage.from_event(self, event))
+        return receipts
