@@ -238,16 +238,30 @@ class IRCMessage(immp.Message):
         elif line.command == "KICK":
             target = await irc.user_from_username(line.args[1])
             text = immp.RichText([immp.Segment("kicked "),
-                                  immp.Segment(target.username, bold=True),
+                                  immp.Segment(target.username, bold=True, mention=target),
                                   immp.Segment(" ({})".format(line.args[2]))])
             action = True
             left.append(target)
         elif line.command == "PRIVMSG":
-            text = line.args[1]
-            match = re.match(r"\x01ACTION ([^\x01]*)\x01", text)
+            plain = line.args[1]
+            match = re.match(r"\x01ACTION ([^\x01]*)\x01", plain)
             if match:
-                text = match.group(1)
+                plain = match.group(1)
                 action = True
+            text = immp.RichText()
+            puppets = {client.nick: user for user, client in irc._puppets.items()}
+            for match in re.finditer(r"[\w\d_\-\[\]{}\|`]+", plain):
+                word = match.group(0)
+                if word in puppets:
+                    target = puppets[word]
+                else:
+                    target = irc.get_user(word)
+                if target:
+                    if len(text) < match.start():
+                        text.append(immp.Segment(plain[len(text):match.start()]))
+                    text.append(immp.Segment(word, mention=target))
+            if len(text) < len(plain):
+                text.append(immp.Segment(plain[len(text):]))
         else:
             raise NotImplementedError
         return immp.SentMessage(id_=Line.next_ts(),
@@ -268,8 +282,7 @@ class Wait:
     line is received.  On failure, a :class:`ValueError` is raised.
     """
 
-    def __init__(self, lines, success, fail, collect):
-        self.lines = lines
+    def __init__(self, success, fail, collect):
         self._success = tuple(success)
         self._fail = tuple(fail)
         self._collect = tuple(collect)
@@ -341,19 +354,25 @@ class IRCClient:
         # Capture answers to pending queries.
         self._waits = []
 
+    async def set_nick(self, value):
+        if self._nicklen:
+            value = value[:self._nicklen]
+        if self._nick == value:
+            return
+        if self._writer:
+            await self._wait(Line("NICK", value),
+                             success=("001", "NICK"),
+                             fail=("431", "432", "433", "436"))
+        else:
+            self._nick = value
+
     @property
     def nick(self):
         return self._nick
 
     @nick.setter
     def nick(self, value):
-        if self._nicklen:
-            value = value[:self._nicklen]
-        if self._nick == value:
-            return
-        self._nick = value
-        if self._writer:
-            self._write(Line("NICK", self._nick))
+        ensure_future(self.set_nick(value))
 
     @property
     def nickmask(self):
@@ -383,7 +402,7 @@ class IRCClient:
             self._writer.write("{}\r\n".format(line).encode())
 
     async def _wait(self, *lines, success=(), fail=(), collect=()):
-        wait = Wait((), success, fail, collect)
+        wait = Wait(success, fail, collect)
         log.debug("Adding wait: %r", wait)
         self._waits.append(wait)
         self._write(*lines)
@@ -425,10 +444,16 @@ class IRCClient:
                     self._nicklen = int(value)
                 elif key == "NETWORK":
                     self.network = value
-        elif line.command == "433":
+        elif line.command in ("431", "432"):
+            # Nick change rejected, revert internal nick to who the server says we are.
+            if self._nick != line.args[0]:
+                log.debug("Reverting failed nick change: %s -> %s", self._nick, line.args[0])
+                self._nick = line.args[0]
+        elif line.command in ("433", "436"):
             # Re-request the current nick with a trailing underscore.
             # Remove characters from the nick if needed to make it fit.
             parsed = line.args[1]
+            log.debug("Nick collision: %s", parsed)
             if len(parsed) < len(self._nick):
                 # We got silently truncated, set the max nick length.
                 self._nicklen = len(parsed)
@@ -465,6 +490,7 @@ class IRCClient:
             old = line.source.split("!", 1)[0]
             new = line.args[0]
             if self._nick == old:
+                log.debug("Updating own nick: %s -> %s", self._nick, new)
                 self._nick = new
             for name, members in list(self.members.items()):
                 if name == old:
@@ -607,19 +633,18 @@ class IRCPlug(immp.Plug):
             await client.disconnect(self.config["quit"])
         self._puppets.clear()
 
+    def get_user(self, nick):
+        return self._client.users.get(nick)
+
     async def user_from_id(self, id_):
         nick = id_.split("!", 1)[0]
         user = await self.user_from_username(nick)
-        if user:
-            return user
-        else:
-            return IRCUser.from_id(self, id_)
+        return user or IRCUser.from_id(self, id_)
 
     async def user_from_username(self, username):
-        try:
-            return self._client.users[username]
-        except KeyError:
-            pass
+        user = self.get_user(username)
+        if user:
+            return user
         for user in await self._client.who(username):
             if user.username == username:
                 return user
@@ -727,7 +752,7 @@ class IRCPlug(immp.Plug):
         else:
             log.debug("Reusing puppet %r for user %r", puppet.nickmask, user)
             if puppet.nick.rstrip("_") != nick:
-                puppet.nick = nick
+                await puppet.set_nick(nick)
             return puppet
         if user.plug and user.plug.network_id == self.network_id:
             for puppet in self._puppets.values():
@@ -764,7 +789,7 @@ class IRCPlug(immp.Plug):
                 lines += self._lines(text, user, True, msg.edited)
             elif isinstance(attach, immp.Message) and attach.text:
                 lines += self._lines(attach.text, attach.user, attach.action, attach.edited)
-        ids = []
+        receipts = []
         if self.config["puppet"] and msg.user:
             client = await self._puppet(msg.user)
             if not await channel.is_private():
@@ -775,5 +800,5 @@ class IRCPlug(immp.Plug):
             line = client.send(channel.source, text)
             sent = await IRCMessage.from_line(self, line)
             self.queue(sent)
-            ids.append(sent.id)
-        return ids
+            receipts.append(sent)
+        return receipts
