@@ -1,6 +1,9 @@
 """
 Connect to a `Slack <https://slack.com>`_ workspace as a bot.
 
+Requirements:
+    `aiohttp <https://aiohttp.readthedocs.io/en/latest/>`_
+
 Config:
     token (str):
         Slack API token for a bot user.
@@ -160,7 +163,7 @@ class _Schema:
 
     members = _api({"members": [str]})
 
-    post = _api({"ts": str})
+    post = _api({"channel": str, "message": message})
 
     upload = _api({"file": file})
 
@@ -560,7 +563,7 @@ class SlackMessage(immp.Message):
             # Slack leaves a leading space in the message text: " set up a reminder..."
             text = text.lstrip()
         if event["thread_ts"] and event["ts"] != event["thread_ts"] and parent:
-            thread = immp.Receipt(event["thread_ts"], immp.Channel(slack, event["channel"]))
+            thread = immp.Receipt(event["thread_ts"], channel)
             # Look for the current reply in the thread, and take the previous message as reply-to.
             last = None
             for entry in await slack.get_replies(thread):
@@ -653,14 +656,14 @@ class SlackMessage(immp.Message):
         event = _Schema.message(json)
         if event["hidden"]:
             # Ignore UI-hidden events (e.g. tombstones of deleted files).
-            raise NotImplementedError
+            raise NotImplementedError("hidden")
         if event["is_ephemeral"]:
             # Ignore user-private messages from Slack (e.g. over quota warnings, link unfurling
             # opt-in prompts etc.) which shouldn't be served to message processors.
-            raise NotImplementedError
+            raise NotImplementedError("ephemeral")
         if event["subtype"] == "file_comment":
             # Deprecated in favour of file threads, but Slack may still emit these.
-            raise NotImplementedError
+            raise NotImplementedError("deprecated")
         channel = immp.Channel(slack, event["channel"])
         if event["subtype"] == "message_deleted":
             id_, at = cls._parse_meta(slack, event)
@@ -673,17 +676,37 @@ class SlackMessage(immp.Message):
         elif event["subtype"] == "message_changed":
             if event["message"]["hidden"]:
                 # In theory this should match event["hidden"], but redefined here just in case.
-                raise NotImplementedError
+                raise NotImplementedError("hidden")
             if event["message"]["text"] == event["previous_message"]["text"]:
                 # Message remains unchanged.  Can be caused by link unfurling (adds an attachment)
                 # or deleting replies (reply is removed from event.replies in new *and old*).
-                raise NotImplementedError
+                raise NotImplementedError("unchanged")
             revision = event["ts"]
             # Original message details are under a nested "message" key.
             return await cls._parse_main(slack, json, event["message"], channel, parent, revision)
         else:
             return await cls._parse_main(slack, json, event, channel, parent)
 
+    @classmethod
+    async def from_post(cls, slack, json):
+        """
+        Convert an API post response :class:`dict` to a :class:`.Message`.
+
+        Args:
+            slack (.SlackPlug):
+                Related plug instance that provides the event.
+            json (dict):
+                Slack API response payload from
+                `postMessage <https://api.slack.com/methods/chat.postMessage#response>`_.
+
+        Returns:
+            .SlackMessage:
+                Parsed message object.
+        """
+        post = _Schema.post(json)
+        channel = immp.Channel(slack, post["channel"])
+        return await cls._parse_main(slack, json, post["message"], channel, True)
+
     @classmethod
     def to_attachment(cls, slack, msg, reply=False):
         """
@@ -975,7 +998,7 @@ class SlackPlug(immp.HTTPOpenable, immp.Plug):
         raise MessageNotFound
 
     async def _post(self, channel, parent, msg):
-        ids = []
+        receipts = []
         uploads = 0
         name = None
         data = {"channel": channel.source, "as_user": True}
@@ -990,28 +1013,29 @@ class SlackPlug(immp.HTTPOpenable, immp.Plug):
         for attach in msg.attachments:
             if isinstance(attach, immp.File):
                 # Upload each file to Slack.
-                data = FormData({"channels": channel.source,
+                form = FormData({"channels": channel.source,
                                  "filename": attach.title or ""})
                 if isinstance(parent.reply_to, immp.Receipt):
                     # Reply directly to the corresponding thread.  Note that thread_ts can be any
                     # message in the thread, it need not be resolved to the parent.
-                    data.add_field("thread_ts", msg.reply_to.id)
+                    form.add_field("thread_ts", msg.reply_to.id)
                     if self.config["thread-broadcast"]:
-                        data.add_field("broadcast", "true")
+                        form.add_field("broadcast", "true")
                 if name:
                     comment = immp.RichText([immp.Segment(name, bold=True, italic=True,
                                                           link=msg.user.link),
                                              immp.Segment(" uploaded this file", italic=True)])
-                    data.add_field("initial_comment", SlackRichText.to_mrkdwn(self, comment))
+                    form.add_field("initial_comment", SlackRichText.to_mrkdwn(self, comment))
                 img_resp = await attach.get_content(self.session)
-                data.add_field("file", img_resp.content, filename="file")
-                upload = await self._api("files.upload", _Schema.upload, data=data)
+                form.add_field("file", img_resp.content, filename="file")
+                upload = await self._api("files.upload", _Schema.upload, data=form)
                 uploads += 1
                 for shared in upload["file"]["shares"].values():
                     if channel.source in shared:
-                        ids += [share["ts"] for share in shared[channel.source]]
-        if len(ids) < uploads:
-            log.warning("Missing some file shares: sent %d, got %d", uploads, len(ids))
+                        ids = [share["ts"] for share in shared[channel.source]]
+                        receipts += [immp.Receipt(id_, channel) for id_ in ids]
+        if len(receipts) < uploads:
+            log.warning("Missing some file shares: sent %d, got %d", uploads, len(receipts))
         rich = None
         if msg.text:
             rich = msg.text.clone()
@@ -1042,14 +1066,14 @@ class SlackPlug(immp.HTTPOpenable, immp.Plug):
             if attachments:
                 data["attachments"] = json_dumps(attachments)
             post = await self._api("chat.postMessage", _Schema.post, data=data)
-            ids.append(post["ts"])
-        return ids
+            receipts.append(await SlackMessage.from_post(self, post))
+        return receipts
 
     async def put(self, channel, msg):
         clone = copy(msg)
         if clone.text:
             clone.text = msg.text.clone()
-        forward_ids = []
+        forwards = []
         for attach in msg.attachments:
             if isinstance(attach, immp.Receipt):
                 # No public API to share a message, rely on archive link unfurling instead.
@@ -1060,13 +1084,13 @@ class SlackPlug(immp.HTTPOpenable, immp.Plug):
                 else:
                     clone.text = immp.RichText([immp.Segment(link)])
             elif isinstance(attach, immp.Message):
-                forward_ids += await self._post(channel, clone, attach)
-        own_ids = await self._post(channel, clone, clone)
-        if forward_ids and not own_ids and msg.user:
+                forwards += await self._post(channel, clone, attach)
+        own = await self._post(channel, clone, clone)
+        if forwards and not own and msg.user:
             # Forwarding a message but no content to show who forwarded it.
             info = immp.Message(user=msg.user, action=True, text="forwarded a message")
-            own_ids += await self._post(channel, msg, info)
-        return forward_ids + own_ids
+            own += await self._post(channel, msg, info)
+        return forwards + own
 
     async def delete(self, sent):
         await self._api("chat.delete", params={"channel": sent.channel.source, "ts": sent.id})
@@ -1118,7 +1142,7 @@ class SlackPlug(immp.HTTPOpenable, immp.Plug):
                 # A new message arrived, push it back to the host.
                 try:
                     sent = await SlackMessage.from_event(self, event)
-                except NotImplementedError:
-                    log.debug("Ignoring message with ts %r", event.get("ts"))
+                except NotImplementedError as e:
+                    log.debug("Ignoring message with ts %r (%s)", event.get("ts"), e.args[0])
                 else:
                     self.queue(sent)
